#include <autoconf.h>
#include <platform.h>
#include <herrno.h>
#include <arch/iomux.h>
#include <i2c_mx25_gpio.h>

struct mx25_gpio_i2c_device i2c_pca9538 = {
	.scl_pin = MX25_PIN_CSPI1_SCLK,
	.sda_pin = MX25_PIN_CSPI1_SS1,
	.wait_usecs = 6,
};

#define PCA9538_ADDR	(0xe0)

#define PCA953X_INPUT		(0)
#define PCA953X_OUTPUT		(1)
#define PCA953X_INVERT		(2)
#define PCA953X_DIRECTION	(3)

static int pca9538_write_reg(u8 reg, u8 val)
{
	u8 head_write = PCA9538_ADDR | I2C_WR_BIT;

	struct i2c_msg msgs[] = {
		[0] = { &head_write, 1, I2C_M_WR },
		[1] = { &reg, 1, I2C_M_WR },
		[2] = { &val, 1, I2C_M_WR },
	};

	return mx25_i2c_gpio_xfer(&i2c_pca9538, msgs, ARRAY_SIZE(msgs));
}

static int pca9538_read_reg(u8 reg, u8 *val)
{
	u8 head_write = PCA9538_ADDR | I2C_WR_BIT;
	u8 head_read =  PCA9538_ADDR | I2C_RD_BIT;

	struct i2c_msg msgs[] = {
		[0] = { &head_write, 1, I2C_M_WR },
		[1] = { &reg, 1, I2C_M_WR },
		[2] = { &head_read, 1, I2C_M_WR | I2C_REPSTART},
		[3] = { val, 1, I2C_M_RD },
	};

	return mx25_i2c_gpio_xfer(&i2c_pca9538, msgs, ARRAY_SIZE(msgs));
}

int pca9538_gpio_direction_input(unsigned offset)
{
	u8 reg;
	int ret;

	ret = pca9538_read_reg(PCA953X_DIRECTION, &reg);
	if (ret)
		return ret;

	reg |= (1u << offset);
	return pca9538_write_reg(PCA953X_DIRECTION, reg);
}

int pca9538_gpio_direction_output(unsigned offset, int val)
{
	u8 reg;
	int ret;

	ret = pca9538_read_reg(PCA953X_OUTPUT, &reg);
	if (ret)
		return ret;

	if (val)
		reg |= (1u << offset);
	else
		reg &= ~(1u << offset);

	ret = pca9538_write_reg(PCA953X_OUTPUT, reg);
	if (ret)
		return ret;


	ret = pca9538_read_reg(PCA953X_DIRECTION, &reg);
	if (ret)
		return ret;

	reg &= ~(1u << offset);
	return pca9538_write_reg(PCA953X_DIRECTION, reg);
}

int pca9538_gpio_get_value(unsigned offset)
{
	u8 reg;
	int ret;

	ret = pca9538_read_reg(PCA953X_INPUT, &reg);
	if (ret)
		return ret;

	return (reg & (1u << offset)) ? 1 : 0;
}
